//
//    File: CorrMon.hh
//

#ifndef CORRMON_HH
#define CORRMON_HH

// #include "CorrMonWindow.hh"

#include <fstream>
#include <string>

// DMT includes
#include "DatEnv.hh"
#include "TrigClient.hh"
#include "MonServer.hh"
#include "Time.hh"
#include "Trend.hh"
#include "FSeries.hh"
#include "TSeries.hh"
#include "Hanning.hh"
#include "FSpectrum.hh"
#include "TrigPar.hh"

// #include "Histogram1.hh"

// ROOT includes
#include "TROOT.h"
#include <TEnv.h>
#include "TApplication.h"
#include "TRootHelpDialog.h"
#include "TLine.h"
#include "TFrame.h"
#include "TGraph.h"
#include "TPad.h"
#include "TPaveText.h"
#include "TPaveLabel.h"
#include "TLatex.h"
#include "TCanvas.h"
#include "TStyle.h"
#include "TString.h"
#include "TH1.h"
#include "TError.h"
#include "HistogramUtil.hh"    



// Includes from Daniel's GUI Template
// -----------------------------------
#include <sys/types.h>
#include <pthread.h>
#include <unistd.h>
#include <iostream>


// Includes from John's ChanList function
// --------------------------------------
// #include "framecpp/FrameH.hh"
// #include "framecpp/FrRawData.hh"
// #include "framecpp/FrAdcData.hh"



namespace html {
  class document;
  class text;
}

#include "html/color.hh"
#include "html/document.hh"
#include "html/table.hh"
#include "html/size.hh"
#include "html/text.hh"
#include "html/writer.hh"
#include "html/font.hh"
#include "html/hline.hh"
#include "html/link.hh"




class Chan {
public:
  Chan() {}
  ~Chan () {}
  char mName[40];
  char mType[8];
  int mPoint;
  float mRate;
  TSeries* mTS;
  FSeries* mFS;
  FSpectrum* mSpec;
};

struct freqEntry {
  double freq;
  char freqString[16];
};






class CorrMon : public DatEnv, TrigClient, MonServer  {


  public:
  CorrMon(int argc, const char* argv[]);
  ~CorrMon();
  virtual void ProcessData(void);
  virtual void Attention (void) { MonServer::Attention(); }



  private:
  inline int RC(int row,int col,int size) {return ((row)+(size)*(col));}   // clapack packing conventions
  void readConfigFile(const char* config);
  void dumpHTML(const char* channel, const char* ofile);
  void corrThreshold();
  void initialiseGraphics();
  void addTitle();
  void addChannelNames();
  void initialisePad(TPad *);
  void fillGraph(int,int);
  void makeEPSFile();
  void writeRhoSq(float,float);
  void openRhoSqFile();
  void closeRhoSqFile();
  void rhoSqTrigger(int,int);
  int chanClean(int,FSeries*);
  void usage();

  std::string htmlDir;
  //  string trendDir;
  std::string outputDir;
  // string trendFile;
  std::string indexFile;
  std::string filePath;
  std::string htmlFile;


  bool mEps;
  bool mDebug;
  bool mLog;
  bool mSummary;
  bool mWindow;
  bool mClean;
  bool mTrigger;
  bool mTrendFlag;
  bool mFirstTime;
  int mPasses;
  std::ofstream rhoSqOFS;
  const char* config;
  float deltaF;
  float dTime;
  int correlationWidth;
  int nAveragingTimes;
  float lowLevelTrigger;
  float highLevelTrigger;
  float threshold;
  float minPlotFreq;
  float maxPlotFreq;

  std::string gpsTime;
  Time::ulong_t currentGPS;    // current GPS time stamp
  Time::ulong_t startGPS;      // start GPS time stamp
  Time::ulong_t liveGPS;       // live time

  int nOffset;
  int nChan;
  int nEnvChan;
  int nRho;
  float mModX2Sum;
  float *rhoSq;
  float **rhoSqPairwise;
  float **avgRhoSqPairwise;
  float **maxRhoSqPairwise;
  unsigned int **monitorCount;
  unsigned int **triggerCount;
  // FSpectrum avgRhoSqPairwiseSpec;
  Chan *chanv;
  Chan cleanedSignal;
  fComplex *cleanedFSData;
  fComplex *mA;
  fComplex *mB;
  int nPlot;
  int nTSBuffer;  // size of dmtviewer TSeries plot 
  size_t mPlotDecimation;
  int nCorrBuffer;  // size of dmt Correlation buffer
  float detailMin;
  float signalRate;
  Trend mTrend;
  int mCount;
  char descriptor[128];
  char startTime[128];
  char trendFile[256];
  std::string startTimeString;
  std::vector<freqEntry> frequencies;
  std::string logFileName;
  std::ofstream logFile;
  std::ofstream summaryFile;
  bool summaryFileExists;
  TSeries *plotTS;
  FSpectrum *plotSpec;
  TSeries *plotCleanedTS;
  FSpectrum *plotCleanedSpec;
  TCanvas *c;
  TPad *corrPad[3];
  TPad *spectrumPad[2];
  TH1F **corrGraph;
  TH1F *spectrumGraph[2];
  std::vector<FSpectrum*> avgRhoSqPairwiseSpec;
  //  FSpectrum** avgRhoSqPairwiseSpec;
 
  TrigPar  mDict;
};     



#endif// CORRMON_HH









