/* -*- mode: c++; c-basic-offset: 4; -*- */
#ifndef TABLE_CELL_HH
#define TABLE_CELL_HH
//#include "format.hh"
#include <string>
#include <stdexcept>

//======================================  Cell class definition
namespace table {
  class Cell {
  public:
    enum cell_type {
      kUnknown,
      kNull,
      kNumeric,
      kString
    };
    explicit Cell(const std::string& str);
    bool operator==(const Cell& c) const;
    bool operator<(const Cell& c) const;
    Cell& operator+=(const Cell& c);
    Cell& operator|=(const Cell& c);
    void append(const std::string& x);
    double getNumeric(void) const throw(std::runtime_error);
    const std::string& getString(void) const;
    cell_type getType(void) const;
    unsigned int getWidth(void) const;
    void setType(void);
    void setNumeric(double x);
  private:
    cell_type mType;
    std::string mContent;
    double mNumeric;
  };

  //=====================================  Inline methods
  inline bool
  Cell::operator==(const Cell& c) const {
      if (mType == kNumeric && c.mType == kNumeric) 
	  return (mNumeric == c.mNumeric);
      return (mContent == c.mContent);
  }

  inline bool
  Cell::operator<(const Cell& c) const {
      if (mType == kNumeric && c.mType == kNumeric) 
	  return (mNumeric < c.mNumeric);
      return (mContent < c.mContent);
  }

  inline unsigned int
  Cell::getWidth(void) const {
      return mContent.size();
  }

  inline const std::string& 
  Cell::getString(void) const {
      return mContent;
  }

  inline Cell::cell_type
  Cell::getType(void) const {
      return mType;
  }
}
#endif // !defined(TABLE_CELL_HH)
